# Generated by roxygen2: do not edit by hand

S3method(summary,depFit)
S3method(summary,indepFit)
export(Chronometer)
export(NonParTrans)
export(ParamCop)
export(QRdepCens)
export(SolveL)
export(SolveLI)
export(SurvDC)
export(TCsim)
export(estimate.cmprsk)
export(fitDepCens)
export(fitIndepCens)
export(pi.surv)
import(MASS)
import(foreach)
import(matrixcalc)
import(mvtnorm)
import(nleqslv)
import(nloptr)
import(numDeriv)
import(parallel)
import(pbivnorm)
import(rafalib)
import(rvinecopulib)
import(stats)
importFrom(EnvStats,base)
importFrom(EnvStats,qlnormTrunc)
importFrom(MASS,mvrnorm)
importFrom(Matrix,bdiag)
importFrom(R6,R6Class)
importFrom(copula,archmCopula)
importFrom(copula,frankCopula)
importFrom(copula,gumbelCopula)
importFrom(copula,normalCopula)
importFrom(copula,pCopula)
importFrom(copula,rCopula)
importFrom(copula,tau)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,foreach)
importFrom(grDevices,dev.off)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(lubridate,now)
importFrom(methods,is)
importFrom(nloptr,nloptr)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(rvinecopulib,ktau_to_par)
importFrom(splines2,bSpline)
importFrom(stats,as.formula)
importFrom(stats,constrOptim)
importFrom(stats,dlnorm)
importFrom(stats,dlogis)
importFrom(stats,dnorm)
importFrom(stats,dweibull)
importFrom(stats,integrate)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,plnorm)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,pweibull)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,survreg)
importFrom(utils,install.packages)
importFrom(utils,tail)
importFrom(utils,write.csv)
