% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CopulaCoxPHmodel_BootstrapMethods.R
\name{summary.depFit}
\alias{summary.depFit}
\title{Summary of \code{depCensoringFit} object}
\usage{
\method{summary}{depFit}(object, ...)
}
\arguments{
\item{object}{Output of \code{\link{fitDepCens}} function}

\item{...}{Further arguments}
}
\value{
Summary of dependent censoring model fit in the form of table
}
\description{
Summary of \code{depCensoringFit} object
}
